﻿using System;
using System.Collections.Generic;

namespace AdrianKousz.Util
{
	public class CmdApp
	{
		IApp<string, string[]> app;
		string[] args;

		public CmdApp(IApp<string, string[]> app, string[] args)
		{
			this.app = app;
			this.args = args;
		}

		public void Run()
		{
			Console.Out.WriteLine(app.GetHeader());
			Console.Out.WriteLine();

			var commands = app.GetCommands();

			var showhelp = true;
			var requestedcmd = "";
			if (args.Length != 0) {
				requestedcmd = args[0].Trim().ToLower();
				showhelp = !commands.ContainsKey(requestedcmd);
			}

			if (showhelp) {
				Console.Out.WriteLine("Available commands are:");
				foreach (var icommand in commands) {
					Console.Out.WriteLine("  {0,-12} {1}", icommand.Key, icommand.Value.GetDescription());
				}
				Console.Out.WriteLine();
				return;
			}

			var cmd = commands[requestedcmd];
			var requireargcount = cmd.GetArgumentCount();

			if (args.Length - 1 < requireargcount) {
				Console.Out.WriteLine("Provide at least {0} arguments for \"{1}\". See help:", requireargcount, requestedcmd);
				Console.Out.WriteLine();
				Console.Out.WriteLine(cmd.GetHelp());
				Console.Out.WriteLine();
				return;
			}

			var cmdargs = new string[args.Length - 1];
			Array.Copy(args, 1, cmdargs, 0, cmdargs.Length);
			cmd.SetArguments(cmdargs);
			cmd.Run();
		}
	}
}
