﻿using System;
using System.IO;
using System.Text;

namespace AdrianKousz.Util
{
	public class ExtendedBinaryReader : BinaryReader
	{
		public bool Strict;

		private const string ErrorChooseMethod = "Choose string writing method";
		private const string ErrorHasNull = "String contains trailing null";
		private const string ErrorHasNoNull = "String does not contain trailing null";

		private readonly char[] nullChar = new char[] { '\0' };
		private readonly Encoding encoding;

		public ExtendedBinaryReader(Stream stream, Encoding enc, bool strict = true) : base(stream, enc)
		{
			encoding = enc;
			Strict = strict;
		}

		public override string ReadString()
		{
			throw new NotSupportedException(ErrorChooseMethod);
		}

		public override bool ReadBoolean()
		{
			return ReadInt32() != 0;
		}

		public string ReadString(int len) { return ReadString(len, false); }
		public string ReadStringInt16() { return ReadString(ReadInt16(), false); }
		public string ReadStringInt32() { return ReadString(ReadInt32(), false); }
		public string ReadZString(int len) { return ReadString(len, true); }
		public string ReadZStringInt16() { return ReadString(ReadInt16(), true); }
		public string ReadZStringInt32() { return ReadString(ReadInt32(), true); }

		private string ReadString(int len, bool expectNull)
		{
			if (len == 0) return "";
			var array = ReadBytes(len);
			var str = encoding.GetString(array);

			var index = -1;
			if (expectNull || Strict) index = str.IndexOf("\0");
			if (Strict && !expectNull && index != -1) throw new FormatException(ErrorHasNull);
			if (Strict && expectNull && index == -1) throw new FormatException(ErrorHasNoNull);

			if (index == -1) return str;
			return str.Substring(0, index);
		}
	}
}
