﻿using System;
using System.IO;
using System.Text;

namespace AdrianKousz.Util
{
	public class ExtendedBinaryWriter : BinaryWriter
	{
		private const string ErrorChooseMethod = "Choose string writing method";

		private readonly Encoding encoding;

		public ExtendedBinaryWriter(Stream stream, Encoding enc) : base(stream, enc)
		{
			this.encoding = enc;
		}

		public override void Write(string value)
		{
			throw new NotSupportedException(ErrorChooseMethod);
		}

		public override void Write(bool value)
		{
			if (value) Write((Int32)1);
			else Write((Int32)0);
		}

		// TODO: What gets actually written here?
		public void WriteZStringRaw(string value) { WriteStringRaw(value + "\0"); }
		public void WriteZStringInt16(string value) { WriteStringInt16(value + "\0"); }
		public void WriteZStringInt32(string value) { WriteStringInt32(value + "\0"); }

		public void WriteStringRaw(string value)
		{
			var array = encoding.GetBytes(value);
			Write(array);
		}

		public void WriteStringInt16(string value)
		{
			var array = encoding.GetBytes(value);
			var len = (Int16)array.Length;
			Write(len);
			Write(array);
		}

		public void WriteStringInt32(string value)
		{
			var array = encoding.GetBytes(value);
			var len = (Int32)array.Length;
			Write(len);
			Write(array);
		}

		public void WriteStringPadded(string value, int len)
		{
			var array = new byte[len];
			encoding.GetBytes(value, 0, value.Length, array, 0);
			Write(array);
		}

		public int GetByteCount(string value)
		{
			return encoding.GetByteCount(value);
		}
	}
}
